@@ -22,69 +22,6 @@ Classes concerned with the screen and drawing things on it.
 import os
 import pygame
 
-#: Size of the lesson box (width and height in tiles).
-LESSON_BOX_SIZE = (8, 3)
-#: Size of the inventory (width and height in tiles).
-INVENTORY_SIZE = (6, 1)
-#: Size of a lines bubble (width and height in tiles).
-LINES_BUBBLE_SIZE = (8, 3)
-
-#: Offset used to position printing of the score.
-SCORE_Y_OFFSET = 1
-#: Offset used to position printing of the lines total.
-LINES_Y_OFFSET = 9
-#: Offset used to position printing of the hi-score.
-HISCORE_Y_OFFSET = 17
-
-#: RGB triplet for black.
-BLACK = (0, 0, 0)
-#: RGB triplet for blue.
-BLUE = (0, 0, 197)
-#: RGB triplet for red.
-RED = (197, 0, 0)
-#: RGB triplet for magenta.
-MAGENTA = (197, 0, 197)
-#: RGB triplet for green.
-GREEN = (0, 198, 0)
-#: RGB triplet for cyan.
-CYAN = (0, 198, 197)
-#: RGB triplet for yellow.
-YELLOW = (197, 198, 0)
-#: RGB triplet for white.
-WHITE = (205, 198, 205)
-#: RGB triplet for bright black.
-BRIGHT_BLACK = (0, 0, 0)
-#: RGB triplet for bright blue.
-BRIGHT_BLUE = (0, 0, 255)
-#: RGB triplet for bright red.
-BRIGHT_RED = (255, 0, 0)
-#: RGB triplet for bright magenta.
-BRIGHT_MAGENTA = (255, 0, 255)
-#: RGB triplet for bright green.
-BRIGHT_GREEN = (0, 255, 0)
-#: RGB triplet for bright cyan.
-BRIGHT_CYAN = (0, 255, 255)
-#: RGB triplet for bright yellow.
-BRIGHT_YELLOW = (255, 255, 0)
-#: RGB triplet for bright white.
-BRIGHT_WHITE = (255, 255, 255)
-
-#: Size of the bounding rectangle of a speech bubble (width and height in
-#: tiles).
-SPEECH_BUBBLE_SIZE = (8, 3)
-#: Transparent colour used in the speech bubble.
-SPEECH_BUBBLE_COLORKEY = (0, 255, 0)
-#: Bit pattern used to open the lip of the speech bubble.
-OPEN_LIP_BYTE = 66 # 01000010
-
-#: Transparent colour used in the skool ink image.
-SKOOL_COLORKEY = (255, 255, 255)
-
-#: Ink colour in font.png (used to create transparency).
-FONT_INK = (0, 1, 2)
-#: Paper colour in font.png (used to create transparency).
-FONT_PAPER = (255, 254, 253)
-
 #: ID of the speech bubble image.
 SPEECH_BUBBLE = 'SPEECH_BUBBLE'
 #: ID of the font image.
@@ -110,18 +47,11 @@ SKOOL_INK = 'SKOOL_INK'
 #: ID of the paper-only play area image.
 SKOOL_PAPER = 'SKOOL_PAPER'
 
-#: Inset (in pixels) of the text window from the left and right edges of a
-#: speech bubble.
-SPEECH_BUBBLE_INSET_X = 4
-#: Inset (in pixels) of the text window from the top edge of a speech bubble.
-SPEECH_BUBBLE_INSET_Y = 4
-
 class Screen:
     """Represents the screen upon which everything is drawn.
 
     :type config: dict
-    :param config: Screen configuration parameters (from the `Screen` section
-                   of the ini file).
+    :param config: Configuration parameters from the ini file.
     :param scale: The scale at which the screen will be drawn.
     :type gallery: :class:`Gallery`
     :param gallery: The gallery of images to use for drawing.
@@ -129,32 +59,54 @@ class Screen:
     :param icon_fname: The filename of the Pyskool icon.
     """
     def __init__(self, config, scale, gallery, title, icon_fname):
-        self.initialise_colours()
-        self.width = config['Width']
+        self.width = config.get('Width', 32)
         self.scale = scale
-        self.screen = pygame.display.set_mode(self.scale_coords((self.width, config['Height'])))
-        self.mode = config['GraphicsMode']
-        self.background = config['Background']
-        self.scroll_fps = config['ScrollFps']
-        self.lesson_box_attr = config['LessonBoxAttribute']
-        self.lesson_box_pos = self.scale_coords(config['LessonBoxPos'])
-        self.score_box_coords = config['ScoreBoxPos']
-        self.score_box_x_offset = config['ScoreBoxXOffset']
-        self.score_box_attribute = config['ScoreBoxAttribute']
-        self.mouse_box_coords = config.get('MouseInventoryPos')
-        self.inventory_coords = config.get('InventoryPos')
-        self.scroll_right_offset = config['ScrollRightOffset']
-        self.scroll_left_offset = config['ScrollLeftOffset']
-        self.scroll_columns = config['ScrollColumns']
+        self.screen = pygame.display.set_mode(self.scale_coords((self.width, config.get('Height', 24))))
+        self.mode = config.get('GraphicsMode', 1)
+        self.background = config.get('Background', 2)
+        self.scroll_fps = config.get('ScrollFps', 20)
+        self.lesson_box_ink = config.get('LessonBoxInk', (0, 198, 197))
+        self.lesson_box_paper = config.get('LessonBoxPaper', (0, 0, 0))
+        self.lesson_box_pos = self.scale_coords(config.get('LessonBoxPos', (12, 21)))
+        self.lesson_box_size = self.scale_coords(config.get('LessonBoxSize', (8, 3)))
+        self.score_box_coords = config.get('ScoreBoxPos', (24, 21))
+        self.score_box_x_offset = config.get('ScoreBoxXOffset', 56)
+        self.score_box_ink = config.get('ScoreBoxInk', (197, 198, 0))
+        self.score_box_paper = config.get('ScoreBoxPaper', (0, 0, 0))
+        self.score_offset = config.get('ScoreOffset', 1)
+        self.lines_offset = config.get('LinesOffset', 9)
+        self.hi_score_offset = config.get('HiScoreOffset', 17)
+        mouse_box_pos = config.get('MouseInventoryPos')
+        self.mouse_box_coords = self.scale_coords(mouse_box_pos) if mouse_box_pos else None
+        inventory_pos = config.get('InventoryPos')
+        self.inventory_coords = self.scale_coords(inventory_pos) if inventory_pos else None
+        self.inventory_size = self.scale_coords(config.get('InventorySize', (0, 0)))
+        self.scroll_right_offset = config.get('ScrollRightOffset', 9)
+        self.scroll_left_offset = config.get('ScrollLeftOffset', 10)
+        self.scroll_columns = config.get('ScrollColumns', 8)
         self.bubble = gallery.get_image(SPEECH_BUBBLE)
-        self.font = Font(gallery.get_image(FONT))
+        self.speech_bubble_size = config.get('SpeechBubbleSize', (8, 3))
+        self.speech_bubble_ink = config.get('SpeechBubbleInk', (0, 0, 0))
+        self.speech_bubble_paper = config.get('SpeechBubblePaper', (205, 198, 205))
+        # Create a colorkey for the speech bubble that is different from the
+        # ink and paper colours
+        red = (self.speech_bubble_ink[0] + 1) % 256
+        if red == self.speech_bubble_paper[0]:
+            red = (red + 1) % 256
+        self.speech_bubble_colorkey = (red, 0, 0)
+        self.open_lip_byte = config.get('OpenLipByte', 66)
+        self.speech_bubble_inset = config.get('SpeechBubbleInset', (4, 4))
+        self.lines_bubble_size = self.scale_coords(config.get('LinesBubbleSize', (8, 3)))
+        self.font = Font(gallery.get_image(FONT), config.get('FontInk', (0, 1, 2)), config.get('FontPaper', (255, 254, 253)))
+        self.skool_colorkey = config.get('SkoolInkKey', (255, 255, 255))
+        self.flash_cycle = config.get('FlashCycle', 10)
 
         # Set the window title and icon, and draw the logo and score box
         if os.path.isfile(icon_fname):
             pygame.display.set_icon(pygame.image.load(icon_fname).convert())
         pygame.display.set_caption(title)
-        self.screen.fill(self.colours[self.background])
-        self.screen.blit(gallery.get_image(LOGO), self.scale_coords(config['LogoPos']))
+        self.screen.fill(self.background)
+        self.screen.blit(gallery.get_image(LOGO), self.scale_coords(config.get('LogoPos', (0, 21))))
         self.screen.blit(gallery.get_image(SCOREBOX), self.scale_coords(self.score_box_coords))
 
     def scale_coords(self, coords):
@@ -164,35 +116,15 @@ class Screen:
         """
         return (8 * self.scale * coords[0], 8 * self.scale * coords[1])
 
-    def initialise_colours(self):
-        """Initialise the colour palette."""
-        self.colours = []
-        self.colours.append(BLACK)
-        self.colours.append(BLUE)
-        self.colours.append(RED)
-        self.colours.append(MAGENTA)
-        self.colours.append(GREEN)
-        self.colours.append(CYAN)
-        self.colours.append(YELLOW)
-        self.colours.append(WHITE)
-        self.colours.append(BRIGHT_BLACK)
-        self.colours.append(BRIGHT_BLUE)
-        self.colours.append(BRIGHT_RED)
-        self.colours.append(BRIGHT_MAGENTA)
-        self.colours.append(BRIGHT_GREEN)
-        self.colours.append(BRIGHT_CYAN)
-        self.colours.append(BRIGHT_YELLOW)
-        self.colours.append(BRIGHT_WHITE)
-
     def initialise_column(self, initial_column, skool_width, eric_x):
         """Set the leftmost column of the play area that will appear on the
         screen when the game starts.
 
         :param initial_column: The initial leftmost column.
-        :param skool_width: The width of the entire play area (in tiles).
+        :param skool_width: The width of the entire play area (in pixels).
         :param eric_x: Eric's initial x-coordinate.
         """
-        self.max_column = skool_width - self.width
+        self.max_column = skool_width // (8 * self.scale) - self.width
         default_initial_column = min(self.max_column, max((eric_x - self.width // 2) // 8 * 8, 0))
         column = min(self.max_column, default_initial_column if initial_column is None else initial_column)
         self.column = self.initial_column = max(0, 8 * (column // 8))
@@ -204,6 +136,17 @@ class Screen:
         self.print_inventory()
         self.print_mice()
 
+    def add_font_character(self, char, offset, width):
+        """Define the location and width of a font character bitmap in the font
+        image.
+
+        :param char: The font character.
+        :param offset: The offset (in pixels) of the font character bitmap from
+                       the left edge of the font image.
+        :param width: The width of the font character bitmap.
+        """
+        self.font.add_character(char, offset, width)
+
     def get_scroll_increment(self, x):
         """Return the direction in which the screen should be scrolled when
         Eric is at a given x-coordinate.
@@ -229,12 +172,12 @@ class Screen:
         :param clock: The clock to use to time the scrolling.
         """
         self.column -= self.width
-        background = pygame.Surface((self.screen.get_width(), 8 * self.scale * skool.get_height()))
+        background = pygame.Surface((self.screen.get_width(), skool.get_height()))
         for n in range(self.width):
             self.column += 1
-            skool.draw()
+            skool.draw(False)
             self.screen.blit(background, (-8 * (n + 1) * self.scale, 0))
-            self.update()
+            self._update()
             clock.tick(self.scroll_fps)
 
     def scroll(self, inc, skool, clock):
@@ -258,7 +201,7 @@ class Screen:
         for i in range(num_cols):
             self.column += scroll_inc
             skool.draw()
-            self.update()
+            self._update()
             clock.tick(self.scroll_fps)
 
     def get_text(self, words, ink, paper):
@@ -270,18 +213,6 @@ class Screen:
         """
         return self.font.render(words, ink, paper)
 
-    def get_ink_and_paper(self, attribute):
-        """Return the ink and paper colour derived from an attribute byte. The
-        ink colour is taken from bits 6 (BRIGHT) and 0-2; the paper colour is
-        taken from bits 3-6.
-
-        :param attribute: The attribute byte.
-        :return: A 2-tuple containing the ink and paper colours.
-        """
-        ink = self.colours[(attribute & 64) // 8 + (attribute & 7)]
-        paper = self.colours[(attribute & 120) // 8]
-        return ink, paper
-
     def print_lesson(self, *text_lines):
         """Print some text in the lesson box.
 
@@ -290,11 +221,10 @@ class Screen:
         while len(text_lines) < 2:
             text_lines.append('')
         line1, line2 = text_lines[:2]
-        lesson_box = pygame.Surface(self.scale_coords(LESSON_BOX_SIZE))
-        ink, paper = self.get_ink_and_paper(self.lesson_box_attr)
-        lesson_box.fill(paper)
-        line1_text = self.get_text(line1, ink, paper)
-        line2_text = self.get_text(line2, ink, paper)
+        lesson_box = pygame.Surface(self.lesson_box_size)
+        lesson_box.fill(self.lesson_box_paper)
+        line1_text = self.get_text(line1, self.lesson_box_ink, self.lesson_box_paper)
+        line2_text = self.get_text(line2, self.lesson_box_ink, self.lesson_box_paper)
         font_height = self.scale * 8
         line1_x = (lesson_box.get_width() - line1_text.get_width()) // 2
         line1_y = (lesson_box.get_height() - 2 * font_height) // 2
@@ -303,9 +233,9 @@ class Screen:
         line2_y = line1_y + font_height
         lesson_box.blit(line2_text, (line2_x, line2_y))
         self.screen.blit(lesson_box, self.lesson_box_pos)
-        self.update()
+        self._update(pygame.Rect(self.lesson_box_pos, lesson_box.get_size()))
 
-    def print_number(self, number, y_offset):
+    def _print_number(self, number, y_offset):
         """Print a number (the score, lines total, or hi-score) in the score
         box.
 
@@ -315,38 +245,37 @@ class Screen:
         """
         if number < 0:
             return
-        ink, paper = self.get_ink_and_paper(self.score_box_attribute)
-        number_text = self.get_text(str(number) if number > 0 else "", ink, paper)
+        number_text = self.get_text(str(number) if number > 0 else "", self.score_box_ink, self.score_box_paper)
         width = 24 * self.scale
         background = pygame.Surface((width, 7 * self.scale))
-        background.fill(paper)
+        background.fill(self.score_box_paper)
         background.blit(number_text, (width - number_text.get_width(), 0))
         coords = self.scale_coords(self.score_box_coords)
         top_left_x = coords[0] + self.score_box_x_offset * self.scale - width
         top_left_y = coords[1] + y_offset * self.scale
         self.screen.blit(background, (top_left_x, top_left_y))
-        self.update()
+        self._update(pygame.Rect((top_left_x, top_left_y), background.get_size()))
 
     def print_score(self, score):
         """Print the score in the score box.
 
         :param score: The score.
         """
-        self.print_number(score, SCORE_Y_OFFSET)
+        self._print_number(score, self.score_offset)
 
     def print_lines(self, lines):
         """Print the lines total in the score box.
 
         :param lines: The lines total.
         """
-        self.print_number(lines, LINES_Y_OFFSET)
+        self._print_number(lines, self.lines_offset)
 
     def print_hi_score(self, hiscore):
         """Print the hi-score in the score box.
 
         :param hiscore: The hi-score.
         """
-        self.print_number(hiscore, HISCORE_Y_OFFSET)
+        self._print_number(hiscore, self.hi_score_offset)
 
     def print_inventory(self, item_images=()):
         """Print the inventory. If no inventory is defined, nothing happens.
@@ -356,16 +285,15 @@ class Screen:
         """
         if not self.inventory_coords:
             return
-        inventory_box = pygame.Surface(self.scale_coords(INVENTORY_SIZE))
-        ink, paper = self.get_ink_and_paper(self.background)
-        inventory_box.fill(paper)
+        inventory_box = pygame.Surface(self.inventory_size)
+        inventory_box.fill(self.background)
         x = 0
         for image in item_images:
             if image:
                 inventory_box.blit(image, (x, 0))
                 x += image.get_width()
-        self.screen.blit(inventory_box, self.scale_coords(self.inventory_coords))
-        self.update()
+        self.screen.blit(inventory_box, self.inventory_coords)
+        self._update(pygame.Rect(self.inventory_coords, inventory_box.get_size()))
 
     def print_mice(self, count=0, mouse_image=None):
         """Print the mouse inventory. If no mouse inventory is defined, nothing
@@ -378,12 +306,11 @@ class Screen:
             return
         max_mice = 8
         mouse_box = pygame.Surface(self.scale_coords((max_mice, 1)))
-        ink, paper = self.get_ink_and_paper(self.background)
-        mouse_box.fill(paper)
+        mouse_box.fill(self.background)
         for x in range(min((count, max_mice))):
             mouse_box.blit(mouse_image, (x * mouse_image.get_width(), 0))
-        self.screen.blit(mouse_box, self.scale_coords(self.mouse_box_coords))
-        self.update()
+        self.screen.blit(mouse_box, self.mouse_box_coords)
+        self._update(pygame.Rect(self.mouse_box_coords, mouse_box.get_size()))
 
     def contains(self, character, full=True):
         """Return whether a character is on-screen.
@@ -396,32 +323,32 @@ class Screen:
         """
         return self.column <= character.x <= self.column + self.width - (3 if full else 1)
 
-    def print_lines_bubble(self, x, y, message, ink_colour, paper_colour):
+    def print_lines_bubble(self, x, y, message, ink, paper):
         """Print a lines message bubble.
 
-        :param x: The x-coordinate of the leftmost column of the bubble.
+        :param x: The x-coordinate of the lines-giver's head.
         :param y: The y-coordinate of the top row of the bubble.
-        :param message: The message to write in the bubble.
-        :param ink_colour: The ink colour of the bubble.
-        :param paper_colour: The paper colour of the bubble.
+        :type message: tuple
+        :param message: The lines of text to write in the bubble.
+        :type ink: RGB triplet
+        :param ink: The ink colour of the bubble.
+        :type paper: RGB triplet
+        :param paper: The paper colour of the bubble.
         """
         bubble_x = 8 * (x // 8) - self.column
         bubble_coords = self.scale_coords((bubble_x, y))
 
-        lines_bubble = pygame.Surface(self.scale_coords(LINES_BUBBLE_SIZE))
-        ink = self.colours[ink_colour]
-        paper = self.colours[paper_colour]
+        lines_bubble = pygame.Surface(self.lines_bubble_size)
         lines_bubble.fill(paper)
-        text1 = self.get_text(message[0], ink, paper)
-        text2 = self.get_text(message[1], ink, paper)
-        text1_x = (lines_bubble.get_width() - text1.get_width()) // 2
-        text1_y = self.scale * 4
-        text2_x = (lines_bubble.get_width() - text2.get_width()) // 2
-        text2_y = text1_y + 8 * self.scale
-        lines_bubble.blit(text1, (text1_x, text1_y))
-        lines_bubble.blit(text2, (text2_x, text2_y))
+        bubble_width = lines_bubble.get_width()
+        y = self.scale * 4
+        for line in message:
+            text_image = self.get_text(line, ink, paper)
+            x = (bubble_width - text_image.get_width()) // 2
+            lines_bubble.blit(text_image, (x, y))
+            y += self.scale * 8
         self.screen.blit(lines_bubble, bubble_coords)
-        self.update()
+        self._update(pygame.Rect(bubble_coords, lines_bubble.get_size()))
 
     def get_bubble(self, words, lip_pos, shift):
         """Create a speech bubble displaying a portion of a message.
@@ -436,75 +363,79 @@ class Screen:
                  bubble image (a `pygame.Surface`), and `done` is `True` if
                  the entire message has been spoken, `False` otherwise.
         """
-        ink = self.colours[0]
-        paper = self.colours[7]
-        bubble = pygame.Surface(self.scale_coords(SPEECH_BUBBLE_SIZE))
-        bubble.fill(SPEECH_BUBBLE_COLORKEY)
-        bubble.set_colorkey(SPEECH_BUBBLE_COLORKEY)
+        bubble = pygame.Surface(self.scale_coords(self.speech_bubble_size))
+        bubble.fill(self.speech_bubble_colorkey)
+        bubble.set_colorkey(self.speech_bubble_colorkey)
         bubble.blit(self.bubble, (0, 0))
         lip = self.bubble.subsurface(self.scale_coords((8, 0)), self.scale_coords((1, 1)))
-        lip_coords = self.scale_coords((lip_pos, SPEECH_BUBBLE_SIZE[1] - 1))
+        lip_coords = self.scale_coords((lip_pos, self.speech_bubble_size[1] - 1))
         bubble.blit(lip, lip_coords)
 
         # Open the lip of the speech bubble
-        open_lip_byte = OPEN_LIP_BYTE
+        open_lip_byte = self.open_lip_byte
         for bit in range(8):
             bit_x = bit * self.scale
-            colour = ink if open_lip_byte & 128 else paper
+            colour = self.speech_bubble_ink if open_lip_byte & 128 else self.speech_bubble_paper
             for r in range(self.scale):
                 for c in range(self.scale):
                     bubble.set_at((lip_coords[0] + bit_x + c, lip_coords[1] - 1 - r), colour)
             open_lip_byte *= 2
 
-        text = self.get_text(words, ink, paper)
+        text = self.get_text(words, self.speech_bubble_ink, self.speech_bubble_paper)
         tile_width = 8 * self.scale
-        min_inset_x = SPEECH_BUBBLE_INSET_X * self.scale
+        min_inset_x = self.speech_bubble_inset[0] * self.scale
         inset_x = min_inset_x - tile_width * min(shift, 0)
-        inset_y = SPEECH_BUBBLE_INSET_Y * self.scale
+        inset_y = self.speech_bubble_inset[1] * self.scale
         text_x = max(shift, 0) * tile_width
-        max_width = tile_width * SPEECH_BUBBLE_SIZE[0] - 2 * min_inset_x
+        max_width = tile_width * self.speech_bubble_size[0] - 2 * min_inset_x
         width = min(min_inset_x + max_width - inset_x, text.get_width() - text_x)
         text_window = text.subsurface((text_x, 0), (width, tile_width))
         bubble.blit(text_window, (inset_x, inset_y))
         return (bubble, width < 0)
 
-    def update(self):
-        """Update the screen."""
-        pygame.display.update()
+    def _update(self, *args):
+        """Update the display.
 
-    def draw(self, skool_images, blackboards, cast, others):
+        :param args: Arguments passed to `pygame.display.update`.
+        """
+        pygame.display.update(*args)
+
+    def draw(self, skool_images, blackboards, cast, others, update):
         """Draw everything on the screen.
 
         :param skool_images: The play area images.
-        :param blackboards: The blackboards (3-tuples, `(x, y, text)`, where
-                            `x` and `y` are the coordinates, and `text` is a
-                            list of images of the blackboard contents).
-        :param cast: The cast (3-tuples, `(x, y, sprite)`, where `x` and `y`
-                     are the coordinates, and `sprite` is an image of a cast
+        :param blackboards: The blackboards (3-tuples, `(x, y, image)`, where
+                            `x` and `y` are the coordinates, and `image` is an
+                            image of the blackboard).
+        :param cast: The cast (3-tuples, `(x, y, image)`, where `x` and `y`
+                     are the coordinates, and `image` is an image of a cast
                      member).
         :param others: Anything else to be drawn (3-tuples, `(x, y, image)`).
+        :param update: Whether to update the screen after drawing.
         """
         if self.mode == 0:
-            self.draw_skool(self.screen, skool_images[0])
-            for x, y, text in blackboards:
-                self.draw_blackboard(self.screen, x, y, text)
-            for x, y, sprite in cast:
-                self.draw_image(self.screen, x, y, sprite)
+            self._draw_skool(self.screen, skool_images[0])
+            for x, y, image in blackboards:
+                self._draw_image(self.screen, x, y, image)
+            for x, y, image in cast:
+                self._draw_image(self.screen, x, y, image)
         else:
             height = skool_images[1].get_height()
             scratch = pygame.Surface((self.width * 8 * self.scale, height))
-            self.draw_skool(scratch, skool_images[1])
-            for x, y, text in blackboards:
-                self.draw_blackboard(scratch, x, y, text)
-            for x, y, sprite in cast:
-                self.draw_image(scratch, x, y, sprite)
-            self.draw_skool(self.screen, skool_images[2])
-            scratch.set_colorkey(SKOOL_COLORKEY)
+            self._draw_skool(scratch, skool_images[1])
+            for x, y, image in blackboards:
+                self._draw_image(scratch, x, y, image)
+            for x, y, image in cast:
+                self._draw_image(scratch, x, y, image)
+            self._draw_skool(self.screen, skool_images[2])
+            scratch.set_colorkey(self.skool_colorkey)
             self.screen.blit(scratch, (0, 0))
         for x, y, image in others:
-            self.draw_image(self.screen, x, y, image)
+            self._draw_image(self.screen, x, y, image)
+        if update:
+            self._update()
 
-    def draw_skool(self, surface, skool):
+    def _draw_skool(self, surface, skool):
         """Draw the skool.
 
         :type surface: `pygame.Surface`
@@ -514,21 +445,7 @@ class Screen:
         """
         surface.blit(skool, self.scale_coords((-self.column, 0)))
 
-    def draw_blackboard(self, surface, x, y, text):
-        """Draw the contents of a blackboard.
-
-        :type surface: `pygame.Surface`
-        :param surface: The surface on which to draw the blackboard contents.
-        :param x: The x-coordinate of the leftmost column of the blackboard.
-        :param y: The y-coordinate of the top row of the blackboard.
-        :type text: `pygame.Surface`
-        :param text: A list of images of the blackboard contents.
-        """
-        for line_no, image in enumerate(text):
-            coords = (self.scale * 8 * (x - self.column), self.scale * (8 * (y + line_no) + 1))
-            surface.blit(image, coords)
-
-    def draw_image(self, surface, x, y, image):
+    def _draw_image(self, surface, x, y, image):
         """Draw an image on a surface.
 
         :type surface: `pygame.Surface`
@@ -548,6 +465,14 @@ class Screen:
         """
         pygame.image.save(self.screen, filename)
 
+    def has_font_char(self, char):
+        """Return whether the skool font contains a bitmap for a given
+        character.
+
+        :param char: The character to look for.
+        """
+        return self.font.has_char(char)
+
 class Gallery:
     """A container for all the images used in a game.
 
@@ -555,7 +480,7 @@ class Gallery:
     :param image_set: The name of the image set.
     :param scale: The desired scale of the images.
     :type images: dict
-    :param images: The images (keyed by image ID).
+    :param images: Key-value pairs (image ID, path) from the `Images` section.
     """
     def __init__(self, images_dir, image_set, scale, images):
         self.images_dir = images_dir
@@ -592,12 +517,19 @@ class Font:
 
     :type image: `pygame.Surface`
     :param image: The font image.
+    :type ink_key: RGB triplet
+    :param ink_key: The ink colour in `font.png` (used to create transparency).
+    :type paper_key: RGB triplet
+    :param paper_key: The paper colour in `font.png` (used to create
+                      transparency).
     """
-    def __init__(self, image):
+    def __init__(self, image, ink_key, paper_key):
         self.image = image
-        self.character_offsets = {}
+        self.ink_key = ink_key
+        self.paper_key = paper_key
+        self.characters = {}
 
-    def add_character_offset(self, char, offset, width):
+    def add_character(self, char, offset, width):
         """Define the location and width of a font character bitmap in the font
         image.
 
@@ -606,14 +538,16 @@ class Font:
                        the left edge of the font image.
         :param width: The width of the font character bitmap.
         """
-        self.character_offsets[char] = (offset, width)
+        self.characters[char] = (offset, width)
 
     def render(self, words, ink, paper):
         """Return an image (a `pygame.Surface`) of a text message written in
         the skool font.
 
         :param words: The message.
+        :param ink: RGB triplet
         :param ink: The desired ink colour.
+        :param paper: RGB triplet
         :param paper: The desired paper colour.
         """
         character_images = []
@@ -621,7 +555,7 @@ class Font:
         height = self.image.get_height()
         scale = height // 8
         for c in words:
-            offset, width = self.character_offsets[c]
+            offset, width = self.characters[c]
             image = self.image.subsurface((scale * offset, 0), (scale * width, height))
             character_images.append(image)
             total_width += width
@@ -634,9 +568,9 @@ class Font:
         paper_surface.fill(paper)
         ink_surface = pygame.Surface((text.get_width(), text.get_height()))
         ink_surface.fill(ink)
-        text.set_colorkey(FONT_INK)
+        text.set_colorkey(self.ink_key)
         ink_surface.blit(text, (0, 0))
-        ink_surface.set_colorkey(FONT_PAPER)
+        ink_surface.set_colorkey(self.paper_key)
         paper_surface.blit(ink_surface, (0, 0))
         return paper_surface
 
@@ -646,4 +580,4 @@ class Font:
 
         :param char: The character to look for.
         """
-        return char in self.character_offsets
+        return char in self.characters
